var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
define("bs-raw", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Tiny buffer class for UTF8 strings.
     * @internal
     *
     * It is a subclass of Uint8Array with the added capacity to encode/decode text from/into string and therefore requires to be initialised with BS.setupEncodeDecoder() as such:
     *
     * ```javascript
     * // Encode & Decode using Buffer class (nodejs)
     * BS.setupEncodeDecoder(
     *   {
     *     encode(s) {
     *     return BS.create(Buffer.from(s || ''))
     *     },
     *   },
     *   {
     *     decode(bs) {
     *     return Buffer.from(bs).toString()
     *     },
     *   }
     * )
     * ```
     *
     * or
     *
     * ```javascript
     * // Encode & Decode using TextEncoder and TextDecoder class (nodejs)
     * // import { TextEncoder, TextDecoder } from 'util' // required in nodejs
     * BS.setupEncodeDecoder(new TextEncoder(), new TextDecoder())
     * ```
     *
     * @remarks
     *
     * Use `BS.create()` to create an instance.
     * `slice()` and `subarray()` will return a BS instance.
     *
     */
    class BS extends Uint8Array {
        /**
         * Test whether this byte string is the start of another one for a given length
         *
         *    const abc = BS.create('abc')
         *    const abcdef = BS.create('abcdef')
         *    abc.isStartOf(abcdef, 3) === true
         *    abc.isStartOf(abcdef, 4) === false
         */
        isStartOf(bs, length) {
            if (bs === this)
                return true;
            if (!bs || length !== this.length)
                return false;
            let i = 0;
            while (i < length)
                if (bs[i] !== this[i++])
                    return false;
            return true;
        }
        /**
         * Test strick equality
         *
         *    const abc1 = BS.create('abc')
         *    const abc2 = BS.create('abc')
         *    abc1 !== abc2
         *    abc1.equals(abc2) === true
         */
        equals(bs) {
            return bs === this || (bs && bs.length === this.length && bs.every((b, i) => b === this[i]));
        }
        /**
         * Extend a byte string with another one
         *
         *    const abc = BS.create('abc')
         *    const def = BS.create('def')
         *    const abcdef = abc.append(def)
         *    abcdef.toString() === 'abcdef'
         */
        append(tail) {
            if (this.length) {
                const length = tail.length + this.length;
                const bs = new BS(length);
                bs.set(this);
                bs.set(tail, this.length);
                return bs;
            }
            else {
                return tail;
            }
        }
        /** converts to string */
        toString(start, length) {
            if (start !== undefined && (start !== 0 || length !== this.byteOffset)) {
                return BS.decode(this.subarray(start, start + (length || this.byteOffset)));
            }
            else {
                return BS.decode(this);
            }
        }
        /** @internal */
        static encode(_input) {
            throw Error('Need to call BS.setupEncodeDecoder(new TextEncoder(), new TextDecoder())');
        }
        /** @internal */
        static decode(_input) {
            throw Error('Need to call BS.setupEncodeDecoder(new TextEncoder(), new TextDecoder())');
        }
        /** Defines encoder and decoder - must be called before use */
        static setupEncodeDecoder(textEncoder, textDecoder) {
            BS.encode = textEncoder.encode.bind(textEncoder);
            BS.decode = textDecoder.decode.bind(textDecoder);
        }
        /** @internal peeks the first few chars for debugging */
        get _() {
            return this.length <= 50 ? this.toString() : `${this.subarray(0, 50)}…`;
        }
        /** @internal singleton reffering to an empty array (used for initialisation of internal variables) */
        static get EMPTY() {
            const empty = new BS();
            Object.defineProperty(BS, 'EMPTY', { value: empty });
            return empty;
        }
    }
    exports.BS = BS;
    /** creates an istance from a string or a Uint8Array */
    BS.create = (input) => {
        const bs = typeof input === 'string' ? BS.encode(input) : input;
        return new BS(bs.buffer, bs.byteOffset, bs.length);
    };
});
define("bs-browser", ["require", "exports", "bs-raw", "util"], function (require, exports, bs_raw_1, util_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.BS = bs_raw_1.BS;
    bs_raw_1.BS.setupEncodeDecoder(new util_1.TextEncoder(), new util_1.TextDecoder());
});
define("bs", ["require", "exports", "bs-raw"], function (require, exports, bs_raw_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.BS = bs_raw_2.BS;
    bs_raw_2.BS.setupEncodeDecoder({
        encode(s) {
            return bs_raw_2.BS.create(Buffer.from(s || ''));
        },
    }, {
        decode(bs) {
            return Buffer.from(bs).toString();
        },
    });
});
define("token", ["require", "exports", "bs"], function (require, exports, bs_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Defininition of the tokens emmited by the parser
     * @public
     */
    var Token;
    (function (Token) {
        /**
         * Represent a start-tag, ie, `<a att="value">`
         * @public
         */
        class StartTag {
            constructor(name, atts, ns, selfclosing) {
                this.name = name;
                if (atts.length)
                    this.atts = atts;
                if (ns.length)
                    this.ns = ns;
                if (selfclosing)
                    this.selfClosing = true;
            }
            /**
             * namespace URI
             *
             * with `<x:a xmlns:x="//uri/x">`, aStartTag.namespaceUri === '//uri/x'
             */
            get namespaceUri() {
                if (this.ns) {
                    const coIndex = this.name.indexOf(0x3a /*:*/);
                    const ns = coIndex > -1
                        ? this.ns.find(({ name }) => { var _a; return (_a = name) === null || _a === void 0 ? void 0 : _a.isStartOf(this.name, coIndex); })
                        : this.ns.find(({ name }) => !name);
                    return ns ? ns.uriString : undefined;
                }
                else {
                    return;
                }
            }
            /**
             * with `<x:a xmlns:x="//uri/x">`, aStartTag.tagName === 'x:a'
             */
            get tagName() {
                return this.name.toString();
            }
            /**
             * with `<x:a xmlns:x="//uri/x">`, aStartTag.localName === 'a'
             */
            get localName() {
                const coIndex = this.name.indexOf(0x3a /*:*/);
                return coIndex > -1 ? this.name.subarray(coIndex + 1).toString() : this.name.toString();
            }
            /**
             * with `<a att="value">`, aStartTag.getAttribute('att') === 'value'
             */
            getAttribute(name) {
                return this.getAttributeFQN(bs_1.BS.create(name));
            }
            /** @internal */
            getAttributeFQN(fqName) {
                const att = this.atts && this.atts.find(({ name }) => name.equals(fqName));
                return att && att.value.toString();
            }
            /**
             * with `<a x:att="value" xmlns:x="//uri/x">`, aStartTag.getAttributeNS('//uri/x', 'att') === 'value'
             */
            getAttributeNS(nsUri, localName) {
                if (!this.atts)
                    return;
                const localNameBS = bs_1.BS.create(localName);
                const ns = this.ns && this.ns.find(({ uriString }) => uriString === nsUri);
                if (ns && ns.name) {
                    const nsName = ns.name;
                    const prefixLen = ns.name.length + 1;
                    const att = this.atts.find(({ name }) => {
                        return (localNameBS.length + prefixLen === name.length &&
                            nsName.every((b, i) => name[i] === b) &&
                            localNameBS.every((b, i) => name[i + prefixLen] === b));
                    });
                    return att && att.value.toString();
                }
                else {
                    return this.getAttributeFQN(localNameBS);
                }
            }
            /** @internal */
            get length() {
                let length = 1 + this.name.length + 1;
                if (this.atts)
                    length += this.atts.length + this.atts.reduce((al, { name, value }) => al + name.length + value.length + 3, 0);
                if (this.ns)
                    length +=
                        this.ns.length + this.ns.reduce((al, { name, uri }) => al + (name ? 6 + name.length : 5) + uri.length + 3, 0);
                if (this.selfClosing)
                    length++;
                return length;
            }
            /** @internal */
            get bs() {
                const length = this.length;
                const bs = new bs_1.BS(length);
                bs[0] = 0x3c; /*<*/
                bs.set(this.name, 1);
                let at = this.name.length + 1;
                if (this.atts) {
                    this.atts.forEach(({ name, value }) => {
                        bs[at++] = 0x20; /* */
                        bs.set(name, at);
                        at += name.length;
                        bs[at++] = 0x3d; /*=*/
                        bs[at++] = 0x22; /*"*/
                        bs.set(value, at);
                        at += value.length;
                        bs[at++] = 0x22; /*"*/
                    });
                }
                if (this.ns) {
                    this.ns.forEach(({ name, uri }) => {
                        bs[at++] = 0x20; /* */
                        bs[at++] = 0x78; /*x*/
                        bs[at++] = 0x6d; /*m*/
                        bs[at++] = 0x6c; /*l*/
                        bs[at++] = 0x6e; /*n*/
                        bs[at++] = 0x73; /*s*/
                        if (name) {
                            bs[at++] = 0x3a; /*:*/
                            bs.set(name, at);
                            at += name.length;
                        }
                        bs[at++] = 0x3d; /*=*/
                        bs[at++] = 0x22; /*"*/
                        bs.set(uri, at);
                        at += uri.length;
                        bs[at++] = 0x22; /*"*/
                    });
                }
                if (this.selfClosing)
                    bs[length - 2] = 0x2f; /*/*/
                bs[length - 1] = 0x3e; /*>*/
                return bs;
            }
            /** return this tag as a string (extra space-like character ommited) */
            toString() {
                return this.bs.toString();
            }
        }
        Token.StartTag = StartTag;
        /**
         * Represent a CDATA node, ie, `<![CDATA[ some content ]]>`
         * @public
         */
        class CDATA {
            constructor(content) {
                this.content = content;
            }
            /** return text as a string */
            get textContent() {
                return this.content.toString();
            }
            /** return this tag as a string */
            toString() {
                return `<![CDATA[${this.content}]]>`;
            }
        }
        Token.CDATA = CDATA;
        /**
         * Represent a Text node
         * @public
         */
        class Text {
            constructor(content) {
                this.content = content;
            }
            /** return text as a string, decoding xml entities */
            toString() {
                return this.content.toString();
            }
            /** return text as a string */
            get textContent() {
                return this.decode();
            }
            decode() {
                let o = 0;
                let at = 0;
                let end = -1;
                const bs = this.content;
                const chunks = [];
                while ((o = bs.indexOf(0x26 /*&*/, at)) > -1 && (end = bs.indexOf(0x3b /*;*/, o + 1)) > -1) {
                    chunks.push(bs.subarray(at, o));
                    const entityLength = end - o - 1;
                    const entity = Text.entities.find(e => {
                        return e.bs.length === entityLength && e.bs.every((b, i) => b === bs[o + 1 + i]);
                    });
                    if (entity) {
                        chunks.push(entity.c);
                    }
                    else if (bs[o + 1] === 0x23 /*#*/) {
                        if (bs[o + 2] === 0x78 /*x*/) {
                            try {
                                chunks.push(String.fromCodePoint(parseInt(bs.subarray(o + 3, end).toString(), 16)));
                            }
                            catch (_) {
                                throw new Error(`Invalid entity codepoint (hex) ${bs.subarray(o, end)}`);
                            }
                        }
                        else {
                            try {
                                chunks.push(String.fromCodePoint(parseInt(bs.subarray(o + 2, end).toString())));
                            }
                            catch (_) {
                                throw new Error(`Invalid entity codepoint (dec) ${bs.subarray(o, end + 1)}`);
                            }
                        }
                    }
                    else {
                        throw new Error(`Unknown entity ${bs.subarray(o, end)}`);
                    }
                    at = end + 1;
                }
                if (chunks.length > 0) {
                    chunks.push(bs.subarray(at, bs.length));
                    return chunks.join('');
                }
                else {
                    return bs.toString();
                }
            }
        }
        /** Supported entities (in addition to hex and dec codepoints)
         *
         * @remarks
         *
         * Default entities: `&amp;`, `&gt;`, `&lt;`, `&quot;` and `&apos;`
         */
        Text.entities = [
            { c: '&', bs: [0x61, 0x6d, 0x70] },
            { c: '<', bs: [0x6c, 0x74] },
            { c: '>', bs: [0x67, 0x74] },
            { c: '"', bs: [0x71, 0x75, 0x6f, 0x74] },
            { c: "'", bs: [0x61, 0x70, 0x6f, 0x73] },
        ];
        Token.Text = Text;
        /**
         * Represent a comment node, ie, `<!-- some comment -->`
         * @public
         */
        class Comment {
            constructor(content) {
                this.content = content;
            }
            toString() {
                return `<!--${this.content}-->`;
            }
            /** return comment as a string */
            get textContent() {
                return this.content.toString();
            }
        }
        Token.Comment = Comment;
        /**
         * Represent a end-tag, ie `</a>`
         * @public
         */
        class EndTag {
            constructor(name) {
                this.name = name;
            }
            toString() {
                return `</${this.name}>`;
            }
        }
        Token.EndTag = EndTag;
    })(Token = exports.Token || (exports.Token = {}));
});
define("ns-resolver", ["require", "exports", "bs"], function (require, exports, bs_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Namespace resolver
     *
     * Representing the mapping of epxected namespaces and their URI's
     *
     * Created using an object whith namespaces as keys and URI's as values
     *
     *     const nsResolver = NsResolver.create({svg: 'http://www.w3.org/2000/svg'})
     */
    class NsResolver extends Array {
        /** creates an instance of `NsResolver` */
        static create(nsResolverMap) {
            const nsResolver = new NsResolver();
            for (let nsName in nsResolverMap) {
                const uriString = nsResolverMap[nsName];
                const trgName = nsName ? bs_2.BS.create(nsName) : undefined;
                nsResolver.push({ srcName: trgName, trgName, uriString, uri: bs_2.BS.create(uriString) });
            }
            return nsResolver;
        }
        /**
         * create a new resolver inheriting this one with possibly additional namespace mappings of the current XML files
         *
         *     const nsResolver = NsResolver.create({
         *        abc: 'uri://abc',
         *        lmn: 'uri://lmn',
         *        '': 'uri://uvw',
         *      })
         *      const nsResolverChild = nsResolver.forChildTag([
         *        { name: BS.create('abd'), uri: BS.create('uri://abc') },
         *        { name: undefined, uri: BS.create('uri://lmn') },
         *        { name: BS.create('uvw'), uri: BS.create('uri://uvw') },
         *      ])
         *      nsResolverChild.resolveTagName(BS.create('abd:tagName1')) === 'abc:tagName1'
         *      nsResolverChild.resolveTagName(BS.create('tagName2')) === 'lmn:tagName2'
         *      nsResolverChild.resolveTagName(BS.create('uvw:tagName3')) === 'tagName3'
         *
         */
        forChildTag(ns) {
            if (!ns || ns.length === 0)
                return this;
            const nsResolver = this.slice();
            ns.forEach(({ name: srcName, uri }) => {
                const entry = nsResolver.find(ns => ns.uri.equals(uri));
                if (entry) {
                    entry.srcName = srcName;
                }
                else {
                    nsResolver.push({ srcName, trgName: srcName, uri, uriString: uri.toString() });
                }
            });
            return nsResolver;
        }
        /** resolve the namespace of a fully qualified tag name */
        resolveTagName(name) {
            const coIndex = name.indexOf(0x3a /*:*/);
            if (coIndex > -1) {
                return this.replaceNs(name, coIndex);
            }
            else {
                // maybe add ns
                const ns = this.find(ns => !ns.srcName);
                if (ns && ns.trgName) {
                    const nsTarget = ns.trgName;
                    const mappedName = new bs_2.BS(name.length + nsTarget.length + 1);
                    mappedName.set(nsTarget);
                    mappedName[nsTarget.length] = 0x3a; /*:*/
                    mappedName.set(name.subarray(coIndex + 1), nsTarget.length + 1);
                    return mappedName;
                }
                else {
                    return name;
                }
            }
        }
        replaceNs(name, coIndex) {
            const ns = this.find(ns => { var _a; return (_a = ns.srcName) === null || _a === void 0 ? void 0 : _a.isStartOf(name, coIndex); });
            if (ns && ns.srcName !== ns.trgName) {
                if (!ns.trgName)
                    return name.subarray(coIndex + 1);
                if (ns.srcName && ns.srcName.length === ns.trgName.length) {
                    ns.trgName.forEach((b, i) => (name[i] = b));
                    return name;
                }
                else {
                    const mappedName = new bs_2.BS(name.length + ns.trgName.length - coIndex);
                    mappedName.set(ns.trgName);
                    mappedName.set(name.subarray(coIndex), ns.trgName.length);
                    return mappedName;
                }
            }
            else {
                return name;
            }
        }
        /** resolve the namespace of a fully qualified attribute name */
        resolveAtts(atts) {
            for (let i = 0; i < atts.length; i++) {
                const { name } = atts[i];
                const coIndex = name.indexOf(0x3a /*:*/);
                if (coIndex !== -1) {
                    atts[i].name = this.replaceNs(name, coIndex);
                }
            }
            return atts;
        }
        /** resolve the namespace set (attributes with 'xmlns' prefix) of a tag */
        resolveNs(tagNs) {
            return tagNs.map(({ name, uri }) => {
                const ns = name ? this.find(({ srcName }) => { var _a; return (_a = srcName) === null || _a === void 0 ? void 0 : _a.equals(name); }) : this.find(({ srcName }) => !srcName);
                return ns
                    ? { name: ns.trgName, uriString: ns.uriString, uri: ns.uri }
                    : { name: name, uriString: uri.toString(), uri: uri };
            });
        }
    }
    exports.NsResolver = NsResolver;
});
define("tokenizer", ["require", "exports", "bs", "token", "ns-resolver"], function (require, exports, bs_3, token_1, ns_resolver_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Produces tokens while scanning the input stream.
     * @public
     *
     * @remarks
     *
     * Will emit (using `nextToken()`) any of StartTag, EndTag, Text, Comment, CDATA or undefined when the stream chunk is exhauseted (the last full tag was scanned).
     * On the next write, scanning resumes taking into account any uncomplete tag from the previous chunk.
     *
     * @example
     * Writing the xml in 2 chunks:
     *
     * ```javascript
     * const tokenizer = new Tokenizer()
     * tokenizer.write('<a><b b1="value b1"><c/>some<d>inn')
     * while ((token = tokenizer.nextToken())) {
     *   // do something
     * }
     * tokenizer.write('er</d>text</b></a>')
     * while ((token = tokenizer.nextToken())) {
     *   // do some more
     * }
     * ```
     *
     */
    class Tokenizer {
        constructor(nsResolverMap) {
            // bs offset
            this.at = 0;
            this.bs = bs_3.BS.EMPTY;
            // state properties
            /** carry over partial names or content to next chunk */
            this.tail = bs_3.BS.EMPTY;
            /** tag name of pending start or end tag */
            this.name = bs_3.BS.EMPTY; // tag name
            /** name of pending attribute */
            this.attName = bs_3.BS.EMPTY;
            /** list of pending attributes */
            this.atts = [];
            /** list of pending name spaces */
            this.ns = [];
            /** state of tokenizer of previous chunk */
            this.state = 0;
            this.seenLength = 0;
            this.nsResolverStack = [];
            // level: distance from root (root = 0)
            this.parentNames = [];
            this.skippingLevel = -1;
            this.skippingEndTag = false;
            if (nsResolverMap) {
                this.nsResolver = ns_resolver_1.NsResolver.create(nsResolverMap);
            }
        }
        /**
         * Write some bytes to be scanned.
         *
         * @param bs - byte sequence as an instance of `BS`, `Buffer` or `string`
         */
        write(bs) {
            this.seenLength += this.bs.length;
            this.at = 0;
            this.bs = bs instanceof bs_3.BS ? bs : bs_3.BS.create(bs);
        }
        get exhausted() {
            return this.bs.length === this.at;
        }
        /**
         * Gets the next token
         *
         * return `undefined` when fully scanned or last tag was incomplete, waiting for more.
         */
        nextToken() {
            const { bs } = this;
            while (this.at < bs.byteLength) {
                switch (this.state) {
                    case 0: {
                        if (this.bs[this.at] === 0x3c /*<*/) {
                            this.state = 1;
                            this.at++;
                        }
                        else {
                            this.state = 30;
                        }
                        continue;
                    }
                    case 1 /*<*/:
                        switch (this.bs[this.at]) {
                            case 0x2f /*/*/:
                                this.state = 20;
                                this.at++;
                                continue;
                            case 0x21 /*!*/: {
                                this.state = 50;
                                this.at++;
                                continue;
                            }
                            case 0x3f /*?*/: {
                                this.state = 70;
                                this.at++;
                                continue;
                            }
                            default: {
                                this.state = 10;
                                continue;
                            }
                        }
                    case 10 /*<...*/: {
                        this.name = this.getName(11);
                        this.ns = [];
                        this.atts = [];
                        continue;
                    }
                    case 11 /*<n ...  */: {
                        if (this.skipSpaceLike())
                            continue;
                        switch (this.bs[this.at]) {
                            case 0x3e /*>*/: {
                                this.at++;
                                this.state = 0;
                                const startTag = this.startTag();
                                if (startTag) {
                                    return startTag;
                                }
                                else {
                                    continue;
                                }
                            }
                            case 0x2f /*/*/: {
                                this.at++;
                                this.state = 19;
                                continue;
                            }
                        }
                        this.state = 12;
                        continue;
                    }
                    case 12 /*<n ...  */: {
                        this.attName = this.getName(13);
                        continue;
                    }
                    case 13 /*<n ...  */: {
                        if (this.skipSpaceLike())
                            continue;
                        if (this.bs[this.at++] === 0x3d /*=*/) {
                            this.state = 14;
                        }
                        else {
                            throw new Error(`Expect = after attribute name (at: ${this.seenLength + this.at})`);
                        }
                        continue;
                    }
                    case 14 /*<n ...= */: {
                        if (this.skipSpaceLike())
                            continue;
                        const q = this.bs[this.at++];
                        if (q === 0x22 /*"*/ || q === 0x27 /*'*/) {
                            this.state = q === 0x22 /*"*/ ? 15 : 16;
                            continue;
                        }
                        else {
                            throw new Error(`Expect quote (at: ${this.seenLength + this.at})`);
                        }
                    }
                    case 15 /*<n ..." */:
                    case 16 /*<n ...' */: {
                        const index = this.bs.indexOf(this.state === 15 ? 0x22 /*"*/ : 0x27 /*'*/, this.at);
                        if (index > -1) {
                            this.state = 11;
                            this.capture(this.at, index);
                            if (this.isAtXmlns(this.attName)) {
                                this.ns.push({
                                    name: this.attName.length === 5 ? undefined : this.attName.slice(6),
                                    uri: this.flushCapture(),
                                });
                            }
                            else {
                                this.atts.push({ name: this.attName, value: this.flushCapture() });
                            }
                            this.at++;
                            continue;
                        }
                        else {
                            this.capture(this.at, this.bs.length);
                            continue;
                        }
                    }
                    case 19 /*<n .../ */: {
                        if (this.bs[this.at++] === 0x3e /*>*/) {
                            this.state = 0;
                            const startTag = this.startTag(true);
                            if (startTag) {
                                return startTag;
                            }
                            else {
                                continue;
                            }
                        }
                        else {
                            throw new Error(`Expect closing of self-closing tag (at: ${this.seenLength + this.at})`);
                        }
                    }
                    case 20 /*</*/: {
                        this.name = this.getName(21);
                        continue;
                    }
                    case 21 /*</...*/: {
                        if (this.skipSpaceLike())
                            continue;
                        if (this.bs[this.at++] === 0x3e /*>*/) {
                            this.state = 0;
                            const endTag = this.endTag();
                            if (endTag) {
                                return endTag;
                            }
                            else {
                                continue;
                            }
                        }
                        else {
                            throw new Error(`Expect closing of end-tag (at: ${this.seenLength + this.at})`);
                        }
                    }
                    case 30 /*...*/: {
                        const index = this.bs.indexOf(0x3c /*<*/, this.at);
                        if (index > -1) {
                            this.state = 0;
                            if (this.isSkipping) {
                                this.at = index;
                                continue;
                            }
                            else {
                                this.capture(this.at, index);
                                return new token_1.Token.Text(this.flushCapture());
                            }
                        }
                        else {
                            this.capture(this.at, this.bs.length);
                            continue;
                        }
                    }
                    case 50 /*<!*/: {
                        switch (this.bs[this.at++]) {
                            case 0x2d /*-*/: {
                                this.state = 80;
                                break;
                            }
                            case 0x5b /*[*/: {
                                this.state = 40;
                                break;
                            }
                            default: {
                                throw new Error(`Unexpected tag (at: ${this.seenLength + this.at})`);
                            }
                        }
                        continue;
                    }
                    case 40 /*<![*/: {
                        if (this.bs[this.at++] === 0x43 /*C*/) {
                            this.state++;
                        }
                        else {
                            throw new Error(`Expecting '<![CDATA[' (at: ${this.seenLength + this.at})`);
                        }
                        continue;
                    }
                    case 41 /*<![C*/: {
                        if (this.bs[this.at++] === 0x44 /*D*/) {
                            this.state++;
                        }
                        else {
                            throw new Error(`Expecting '<![CDATA[' (at: ${this.seenLength + this.at})`);
                        }
                        continue;
                    }
                    case 42 /*<![CD*/: {
                        if (this.bs[this.at++] === 0x41 /*A*/) {
                            this.state++;
                        }
                        else {
                            throw new Error(`Expecting '<![CDATA[' (at: ${this.seenLength + this.at})`);
                        }
                        continue;
                    }
                    case 43 /*<![CDA*/: {
                        if (this.bs[this.at++] === 0x54 /*T*/) {
                            this.state++;
                        }
                        else {
                            throw new Error(`Expecting '<![CDATA[' (at: ${this.seenLength + this.at})`);
                        }
                        continue;
                    }
                    case 44 /*<![CDAT*/: {
                        if (this.bs[this.at++] === 0x41 /*A*/) {
                            this.state++;
                        }
                        else {
                            throw new Error(`Expecting '<![CDATA[' (at: ${this.seenLength + this.at})`);
                        }
                        continue;
                    }
                    case 45 /*<![CDATA*/: {
                        if (this.bs[this.at++] === 0x5b /*[*/) {
                            this.state++;
                        }
                        else {
                            throw new Error(`Expecting '<![CDATA[' (at: ${this.seenLength + this.at})`);
                        }
                        continue;
                    }
                    case 46 /*<![CDATA[*/: {
                        // in search for a ']]'
                        this.startingContentCapture(0x5d /*]*/);
                        continue;
                    }
                    case 47 /*may be ending CDATA `<![CDATA[...]`*/: {
                        this.mayBeEndingContentCapture(0x5d /*]*/);
                        continue;
                    }
                    case 48 /*<![CDATA[...]]*/: {
                        if (this.hasContentCaputreEnded(0x5d /*]*/)) {
                            return new token_1.Token.CDATA(this.flushCaptureContent());
                        }
                        else {
                            continue;
                        }
                    }
                    case 80 /*<!-*/: {
                        if (this.bs[this.at++] === 0x2d /*-*/) {
                            this.state++;
                        }
                        else {
                            throw new Error(`Expecting <!-- (at: ${this.seenLength + this.at})`);
                        }
                        continue;
                    }
                    case 81 /*<!--*/: {
                        this.startingContentCapture(0x2d /*-*/);
                        continue;
                    }
                    case 82 /*may be ending comment `<!--...-`*/: {
                        this.mayBeEndingContentCapture(0x2d /*-*/);
                        continue;
                    }
                    case 83 /*<!--...--*/: {
                        if (this.hasContentCaputreEnded(0x2d /*-*/)) {
                            return new token_1.Token.Comment(this.flushCaptureContent());
                        }
                        else {
                            continue;
                        }
                    }
                    case 70 /*<?*/: {
                        const bs = this.bs;
                        let off = this.at;
                        const end = bs.indexOf(0x3f /*?*/, off);
                        if (end > -1) {
                            this.state++;
                            this.at = end + 1;
                        }
                        else {
                            this.at = this.bs.length;
                        }
                        continue;
                    }
                    case 71 /*<?*/: {
                        this.state = bs[this.at++] === 0x3e /*>*/ ? 0 : 70;
                        continue;
                    }
                }
            }
            return;
        }
        /**
         * When called, the tokenizer will skip decoding the child nodes of the last tag.
         * But scanning will resume on its end-tag. This end-tag will not be emitted when called with `true` (`tokenizer.skipChildNodes(true)`)
         * This is a performance optimization as there's no need to decode text, attribute values...
         *
         * @example
         * Skipping nodes
         *
         * ```javascript
         * tokenizer.write('<a><b att="value"><c/>some<d>inner</d>text</b></a>')
         * const aStart = tokenizer.nextToken()
         * const bStart = tokenizer.nextToken()
         * bStart.getAttribute('att') === 'value'
         * tokenizer.skipChildNodes()
         * const bEnd = tokenizer.nextToken()
         * bEnd.toString() === '</b>'
         * ```
         */
        skipChildNodes(skippingEndTag = false) {
            this.skippingLevel = this.parentNames.length;
            this.skippingEndTag = skippingEndTag;
        }
        /** Capuring a name */
        getName(nextState) {
            const { at: start, bs } = this;
            let b = bs[start];
            let at = start;
            //                        ' '          '>'           '/'           'n'           'r'           't'           '='
            while (b !== undefined &&
                b !== 0x20 &&
                b !== 0x3e &&
                b !== 0x2f &&
                b !== 0x0a &&
                b !== 0x0d &&
                b !== 0x09 &&
                b !== 0x3d) {
                b = bs[++at];
            }
            if (start === at && this.tail.length === 0)
                throw new Error(`Expecting name (at: ${this.seenLength + this.at})`);
            const name = this.tail.append(this.bs.subarray(start, at));
            this.at = at;
            if (b !== undefined) {
                this.state = nextState;
                this.tail = bs_3.BS.EMPTY;
            }
            else {
                this.tail = name;
            }
            return name;
        }
        /** Capuring a sequence */
        capture(start, end) {
            this.tail = this.isSkipping ? bs_3.BS.EMPTY : this.tail.append(this.bs.subarray(start, end));
            this.at = end;
        }
        /** Capuring content ending with a given byte */
        startingContentCapture(b) {
            // in search for the first char of a '-->' or ']]>'
            let off = this.at;
            const end = this.bs.indexOf(b, off);
            if (end > -1) {
                this.capture(this.at, end + 1);
                this.state++;
            }
            else {
                this.capture(this.at, this.bs.length);
            }
        }
        /** detecting the possible end of content */
        mayBeEndingContentCapture(b) {
            this.state += this.bs[this.at] === b ? 1 : -1;
            this.capture(this.at, this.at + 1);
        }
        /** detecting the actual end of content */
        hasContentCaputreEnded(b) {
            switch (this.bs[this.at]) {
                case 0x3e /*>*/: {
                    this.state = 0;
                    this.at++;
                    return !this.isSkipping;
                }
                case b: {
                    this.capture(this.at, this.at + 1);
                    return false;
                }
                default: {
                    this.capture(this.at, this.at + 1);
                    this.state -= 2;
                    return false;
                }
            }
        }
        /** returns the catpure (value) */
        flushCapture() {
            const capture = this.tail;
            this.tail = bs_3.BS.EMPTY;
            return capture;
        }
        /** returns the content of comment or CDATA */
        flushCaptureContent() {
            const content = this.tail.subarray(0, this.tail.length - 2);
            this.tail = bs_3.BS.EMPTY;
            return content;
        }
        /** if skipping this token */
        get isSkipping() {
            return this.skippingLevel !== -1 && this.skippingLevel <= this.parentNames.length;
        }
        /** creates a StartTag */
        startTag(selfclosing) {
            if (!selfclosing)
                this.parentNames.push(this.name);
            if (this.isSkipping)
                return;
            if (this.nsResolver) {
                this.nsResolverStack.push(this.nsResolver);
                this.nsResolver = this.nsResolver.forChildTag(this.ns);
                return new token_1.Token.StartTag(this.nsResolver.resolveTagName(this.name), this.nsResolver.resolveAtts(this.atts), this.nsResolver.resolveNs(this.ns), selfclosing);
            }
            else {
                return new token_1.Token.StartTag(this.name, this.atts, this.ns.map(ns => ({ name: ns.name, uri: ns.uri, uriString: ns.uri.toString() })), selfclosing);
            }
        }
        /** creates an EndTag */
        endTag() {
            while (this.parentNames.length > 0) {
                // should it generate EndTag for unproperly closed tags?
                const prevName = this.parentNames.pop();
                if (prevName && this.name.equals(prevName))
                    break;
            }
            if (this.isSkipping)
                return;
            this.skippingLevel = -1;
            if (this.skippingEndTag) {
                this.skippingEndTag = false;
                return;
            }
            const endTag = this.nsResolver
                ? new token_1.Token.EndTag(this.nsResolver.resolveTagName(this.name))
                : new token_1.Token.EndTag(this.name);
            if (this.nsResolver)
                this.nsResolver = this.nsResolverStack.pop();
            return endTag;
        }
        /** skipping spaces, ie after names */
        skipSpaceLike() {
            const bs = this.bs;
            let b = bs[this.at];
            while (b !== undefined && (b === 0x20 || b === 0xa || b === 0xd || b === 0x9))
                b = bs[++this.at];
            return b === undefined;
        }
        /** testing 'XMLNS' or 'xmlns' sequence */
        isAtXmlns(bs) {
            return (bs.length >= 5 && this.isAt_xmlns(bs)) || this.isAt_XMLNS(bs);
        }
        /** testing 'xmlns' sequence */
        isAt_xmlns(bs) {
            return bs[0] === 0x78 && bs[1] === 0x6d && bs[2] === 0x6c && bs[3] === 0x6e && bs[4] === 0x73;
        }
        /** testing 'XMLNS' sequence */
        isAt_XMLNS(bs) {
            return bs[0] === 0x58 && bs[1] === 0x4d && bs[2] === 0x4c && bs[3] === 0x4e && bs[4] === 0x53;
        }
    }
    exports.Tokenizer = Tokenizer;
});
define("parser/xml-like", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Representation of an xml element
     * @beta
     *
     */
    class XmlElement {
        constructor(startTag) {
            this.startTag = startTag;
            /** Children */
            this.children = [];
        }
        /**
         * {@inheritDoc Token.StartTag.getAttribute}
         */
        getAttribute(name) {
            return this.startTag.getAttribute(name);
        }
        /** see {@link Token.StartTag.tagName} */
        get tagName() {
            return this.startTag.tagName;
        }
        /** string as xml */
        toString() {
            return this.startTag.selfClosing
                ? this.startTag.toString()
                : `${this.startTag}${this.children.length ? this.children.join('') : ''}</${this.startTag.tagName}>`;
        }
    }
    exports.XmlElement = XmlElement;
    /**
     * Parser producing a representation of an xml element see class `XmlElement`
     * @beta
     */
    exports.XmlElementParser = () => {
        const parser = {
            onStart(startTag, parent) {
                const element = new XmlElement(startTag);
                parent.children.push(element);
                return element;
            },
            onText(text, ctx) {
                ctx.children.push(text);
            },
            onChild() {
                return this;
            },
        };
        return {
            onStart: (startTag) => new XmlElement(startTag),
            onEnd(ctx) {
                return ctx;
            },
            onText: parser.onText,
            onChild() {
                return parser;
            },
        };
    };
});
define("parser/to-object", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.createObject = (startTag) => {
        const object = {};
        startTag.atts &&
            startTag.atts.forEach(att => {
                object[att.name.toString()] = att.value.toString();
            });
        return object;
    };
    /**
     * Simplified, opiniated, representation of an xml element
     *
     * Produce an object where attributes are properties (string) of the object and
     * children element are arrays of objects.
     *
     * @remarks
     * - Fully qualified names are used for property names.
     * - All texts are collected in property `#text`
     */
    class XmlToObjectParser {
        onStart(startTag, parentCtx) {
            const object = exports.createObject(startTag);
            const tagName = startTag.tagName;
            const collection = parentCtx[tagName] || (parentCtx[tagName] = []);
            collection.push(object);
        }
        onText(text, ctx) {
            ctx['#text'] = ctx['#text'] ? ctx['#text'] + text.textContent : text.textContent;
        }
        onChild() {
            return this;
        }
    }
    exports.XmlToObjectParser = XmlToObjectParser;
});
define("parser/index", ["require", "exports", "bs", "parser/xml-like", "parser/to-object"], function (require, exports, bs_4, XmlLike, to_object_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    XmlLike = __importStar(XmlLike);
    /**
     * Parsing XML Element
     * @public
     */
    var Parser;
    (function (Parser) {
        /**
         * Helps with intermediate nodes of a path like `a/b/c`.
         *
         * When calling docParser.on(`a/b/c`, myParser), `c` will be associated with the target parser but `a` and `b` will be associatiated with `PassThrough`
         */
        class PassThrough {
            constructor(names, parser) {
                this.names = names;
                this.parser = parser;
                this.passThrough = true;
            }
            static on(path, parser) {
                return new PassThrough(path.split('/').map(tagName => bs_4.BS.create(tagName)), parser);
            }
            onChild(startTag) {
                const name = this.names[0];
                return (name.equals(startTag.name) &&
                    (this.names.length > 1 ? new PassThrough(this.names.slice(1), this.parser) : this.parser));
            }
        }
        Parser.PassThrough = PassThrough;
        /**
         * Switches to the first parser amongst a list of possible parsers
         */
        class Switch {
            constructor(then = []) {
                this.then = then;
            }
            static create() {
                return new Switch();
            }
            /**
             * Adds a parser to the list
             * @param parser-parser to be added
             */
            case(parser) {
                this.then.push(parser);
                return this;
            }
            /**
             * Returns the first parser matching the start tag
             * @param startTag-current start-tag
             */
            onChild(startTag) {
                const matchingPassThrough = [];
                const childParsers = this.then;
                for (let i = 0; i < childParsers.length; i++) {
                    const childParser = childParsers[i];
                    const matching = childParser.onChild && childParser.onChild(startTag);
                    if (matching) {
                        if (matching.passThrough) {
                            matchingPassThrough.push(matching);
                        }
                        else {
                            return matching;
                        }
                    }
                }
                return matchingPassThrough.length > 0 && new Switch(matchingPassThrough);
            }
        }
        Parser.Switch = Switch;
        /**
         * Returns parser to generate a simple object representation of an xml element
         *
         * @beta
         */
        function getXmlToObjectParser() {
            const parser = new to_object_1.XmlToObjectParser();
            return {
                onStart: to_object_1.createObject,
                onEnd: (object) => object,
                onText: parser.onText,
                onChild: () => parser,
            };
        }
        Parser.getXmlToObjectParser = getXmlToObjectParser;
        /** Representation of an xml element */
        Parser.XmlElement = XmlLike.XmlElement;
        /** Returns a object representation based on XmlElement */
        Parser.XmlElementParser = XmlLike.XmlElementParser;
    })(Parser = exports.Parser || (exports.Parser = {}));
});
define("index", ["require", "exports", "tokenizer", "document-parser", "parser/index", "token"], function (require, exports, tokenizer_1, document_parser_1, parser_1, token_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.Tokenizer = tokenizer_1.Tokenizer;
    exports.DocumentParser = document_parser_1.DocumentParser;
    exports.Parser = parser_1.Parser;
    exports.Token = token_2.Token;
});
// export { BS } from './bs'
define("document-parser", ["require", "exports", "index", "parser/index"], function (require, exports, _1, parser_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Defines a document parser.
     * @public
     */
    class DocumentParser {
        constructor(tokenizer) {
            this.tokenizer = tokenizer;
        }
        /**
         * Defines how a set of nodes should be parsed
         * @public
         * @param path - a node name of a path to a node (similar to file path, this is not a xpath)
         * @param parser - a parser to be applied
         */
        on(path, parser) {
            if (this.openElement) {
                if (this.openElement.parser instanceof parser_2.Parser.Switch) {
                    this.openElement.parser.case(parser_2.Parser.PassThrough.on(path, parser));
                }
                else {
                    this.openElement.parser = parser_2.Parser.Switch.create()
                        .case(this.openElement.parser)
                        .case(parser_2.Parser.PassThrough.on(path, parser));
                }
            }
            else {
                this.openElement = {
                    parser: parser_2.Parser.PassThrough.on(path, parser),
                    ctx: undefined,
                };
            }
            return this.openElement.parser;
        }
        /**
         * Defines, as for `on()` how the whole document should be parsed
         * @param parser - parser to be applied
         */
        onRoot(parser) {
            this.openElement = {
                parser: {
                    onChild(_) {
                        return parser;
                    },
                },
                ctx: undefined,
            };
        }
        /**
         * Writes xml chunks to the parser
         * @remarks
         * Before writing a new chunk the current one should be exhausted (all tokens have been processed, this means {@link DocumentParser.next} has returned `undefined`)
         * @param chunk- byte sequence (utf8)
         */
        write(chunk) {
            this.tokenizer.write(chunk);
        }
        /**
         * Getting the next element
         */
        next() {
            var _a;
            let token;
            while (this.openElement && (token = this.tokenizer.nextToken())) {
                if (token instanceof _1.Token.StartTag) {
                    const ctx = this.resolveTag(this.openElement, token);
                    if (ctx) {
                        // selfClosing
                        return ctx;
                    }
                }
                else if (token instanceof _1.Token.EndTag) {
                    const ctx = this.openElement.parser.onEnd &&
                        this.openElement.parser.onEnd(this.openElement.ctx, (_a = this.openElement.parent) === null || _a === void 0 ? void 0 : _a.ctx);
                    this.openElement = this.openElement.parent;
                    if (ctx) {
                        return ctx;
                    }
                }
                else if (token instanceof _1.Token.Text || token instanceof _1.Token.CDATA) {
                    this.openElement.parser.onText && this.openElement.parser.onText(token, this.openElement.ctx);
                }
            }
            return undefined;
        }
        resolveTag(openElement, startTag) {
            const parser = openElement.parser.onChild ? openElement.parser.onChild(startTag) : openElement.parser;
            if (parser) {
                const ctx = parser.onStart && parser.onStart(startTag, openElement.ctx);
                if (!startTag.selfClosing && (ctx === false || parser.skipChildNodes)) {
                    this.tokenizer.skipChildNodes();
                }
                if (startTag.selfClosing) {
                    return parser.onEnd && parser.onEnd(ctx, openElement.ctx);
                }
                else {
                    this.openElement = {
                        parent: openElement,
                        parser,
                        ctx: ctx || openElement.ctx,
                    };
                }
            }
            else if (!startTag.selfClosing) {
                this.tokenizer.skipChildNodes(true);
            }
        }
    }
    exports.DocumentParser = DocumentParser;
});
//# sourceMappingURL=index.amd.js.map