"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const bs_1 = require("../bs");
const XmlLike = __importStar(require("./xml-like"));
const to_object_1 = require("./to-object");
/**
 * Parsing XML Element
 * @public
 */
var Parser;
(function (Parser) {
    /**
     * Helps with intermediate nodes of a path like `a/b/c`.
     *
     * When calling docParser.on(`a/b/c`, myParser), `c` will be associated with the target parser but `a` and `b` will be associatiated with `PassThrough`
     */
    class PassThrough {
        constructor(names, parser) {
            this.names = names;
            this.parser = parser;
            this.passThrough = true;
        }
        static on(path, parser) {
            return new PassThrough(path.split('/').map(tagName => bs_1.BS.create(tagName)), parser);
        }
        onChild(startTag) {
            const name = this.names[0];
            return (name.equals(startTag.name) &&
                (this.names.length > 1 ? new PassThrough(this.names.slice(1), this.parser) : this.parser));
        }
    }
    Parser.PassThrough = PassThrough;
    /**
     * Switches to the first parser amongst a list of possible parsers
     */
    class Switch {
        constructor(then = []) {
            this.then = then;
        }
        static create() {
            return new Switch();
        }
        /**
         * Adds a parser to the list
         * @param parser-parser to be added
         */
        case(parser) {
            this.then.push(parser);
            return this;
        }
        /**
         * Returns the first parser matching the start tag
         * @param startTag-current start-tag
         */
        onChild(startTag) {
            const matchingPassThrough = [];
            const childParsers = this.then;
            for (let i = 0; i < childParsers.length; i++) {
                const childParser = childParsers[i];
                const matching = childParser.onChild && childParser.onChild(startTag);
                if (matching) {
                    if (matching.passThrough) {
                        matchingPassThrough.push(matching);
                    }
                    else {
                        return matching;
                    }
                }
            }
            return matchingPassThrough.length > 0 && new Switch(matchingPassThrough);
        }
    }
    Parser.Switch = Switch;
    /**
     * Returns parser to generate a simple object representation of an xml element
     *
     * @beta
     */
    function getXmlToObjectParser() {
        const parser = new to_object_1.XmlToObjectParser();
        return {
            onStart: to_object_1.createObject,
            onEnd: (object) => object,
            onText: parser.onText,
            onChild: () => parser,
        };
    }
    Parser.getXmlToObjectParser = getXmlToObjectParser;
    /** Representation of an xml element */
    Parser.XmlElement = XmlLike.XmlElement;
    /** Returns a object representation based on XmlElement */
    Parser.XmlElementParser = XmlLike.XmlElementParser;
})(Parser = exports.Parser || (exports.Parser = {}));
//# sourceMappingURL=index.js.map