"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createObject = (startTag) => {
    const object = {};
    startTag.atts &&
        startTag.atts.forEach(att => {
            object[att.name.toString()] = att.value.toString();
        });
    return object;
};
/**
 * Simplified, opiniated, representation of an xml element
 *
 * Produce an object where attributes are properties (string) of the object and
 * children element are arrays of objects.
 *
 * @remarks
 * - Fully qualified names are used for property names.
 * - All texts are collected in property `#text`
 */
class XmlToObjectParser {
    onStart(startTag, parentCtx) {
        const object = exports.createObject(startTag);
        const tagName = startTag.tagName;
        const collection = parentCtx[tagName] || (parentCtx[tagName] = []);
        collection.push(object);
    }
    onText(text, ctx) {
        ctx['#text'] = ctx['#text'] ? ctx['#text'] + text.textContent : text.textContent;
    }
    onChild() {
        return this;
    }
}
exports.XmlToObjectParser = XmlToObjectParser;
//# sourceMappingURL=to-object.js.map