"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Representation of an xml element
 * @beta
 *
 */
class XmlElement {
    constructor(startTag) {
        this.startTag = startTag;
        /** Children */
        this.children = [];
    }
    /**
     * {@inheritDoc Token.StartTag.getAttribute}
     */
    getAttribute(name) {
        return this.startTag.getAttribute(name);
    }
    /** see {@link Token.StartTag.tagName} */
    get tagName() {
        return this.startTag.tagName;
    }
    /** string as xml */
    toString() {
        return this.startTag.selfClosing
            ? this.startTag.toString()
            : `${this.startTag}${this.children.length ? this.children.join('') : ''}</${this.startTag.tagName}>`;
    }
}
exports.XmlElement = XmlElement;
/**
 * Parser producing a representation of an xml element see class `XmlElement`
 * @beta
 */
exports.XmlElementParser = () => {
    const parser = {
        onStart(startTag, parent) {
            const element = new XmlElement(startTag);
            parent.children.push(element);
            return element;
        },
        onText(text, ctx) {
            ctx.children.push(text);
        },
        onChild() {
            return this;
        },
    };
    return {
        onStart: (startTag) => new XmlElement(startTag),
        onEnd(ctx) {
            return ctx;
        },
        onText: parser.onText,
        onChild() {
            return parser;
        },
    };
};
//# sourceMappingURL=xml-like.js.map