import { BS } from './bs';
export declare type Attribute = {
    name: BS;
    value: BS;
};
export declare type Ns = {
    name?: BS;
    uriString: string;
    uri: BS;
};
/**
 * Defininition of the tokens emmited by the parser
 * @public
 */
export declare namespace Token {
    /** @internal */
    type Token = Token.EndTag | Token.StartTag | Token.Text | Token.CDATA | Token.Comment;
    /**
     * Represent a start-tag, ie, `<a att="value">`
     * @public
     */
    class StartTag {
        readonly name: BS;
        constructor(name: BS, atts: Attribute[], ns: Ns[], selfclosing?: true);
        /** @internal */
        readonly atts?: {
            name: BS;
            value: BS;
        }[];
        /** @internal */
        readonly ns?: Ns[];
        /** self-closing start-tag, like `<a/>` */
        readonly selfClosing?: true;
        /**
         * namespace URI
         *
         * with `<x:a xmlns:x="//uri/x">`, aStartTag.namespaceUri === '//uri/x'
         */
        get namespaceUri(): string | undefined;
        /**
         * with `<x:a xmlns:x="//uri/x">`, aStartTag.tagName === 'x:a'
         */
        get tagName(): string;
        /**
         * with `<x:a xmlns:x="//uri/x">`, aStartTag.localName === 'a'
         */
        get localName(): string;
        /**
         * with `<a att="value">`, aStartTag.getAttribute('att') === 'value'
         */
        getAttribute(name: string): string | undefined;
        /** @internal */
        getAttributeFQN(fqName: BS): string | undefined;
        /**
         * with `<a x:att="value" xmlns:x="//uri/x">`, aStartTag.getAttributeNS('//uri/x', 'att') === 'value'
         */
        getAttributeNS(nsUri: string, localName: string): string | undefined;
        /** @internal */
        get length(): number;
        /** @internal */
        get bs(): BS;
        /** return this tag as a string (extra space-like character ommited) */
        toString(): string;
    }
    /**
     * Represent a CDATA node, ie, `<![CDATA[ some content ]]>`
     * @public
     */
    class CDATA {
        readonly content: BS;
        constructor(content: BS);
        /** return text as a string */
        get textContent(): string;
        /** return this tag as a string */
        toString(): string;
    }
    /**
     * Represent a Text node
     * @public
     */
    class Text {
        readonly content: BS;
        constructor(content: BS);
        /** return text as a string, decoding xml entities */
        toString(): string;
        /** return text as a string */
        get textContent(): string;
        private decode;
        /** Supported entities (in addition to hex and dec codepoints)
         *
         * @remarks
         *
         * Default entities: `&amp;`, `&gt;`, `&lt;`, `&quot;` and `&apos;`
         */
        static entities: {
            c: string;
            bs: number[];
        }[];
    }
    /**
     * Represent a comment node, ie, `<!-- some comment -->`
     * @public
     */
    class Comment {
        readonly content: BS;
        constructor(content: BS);
        toString(): string;
        /** return comment as a string */
        get textContent(): string;
    }
    /**
     * Represent a end-tag, ie `</a>`
     * @public
     */
    class EndTag {
        readonly name: BS;
        constructor(name: BS);
        toString(): string;
    }
}
